/* MPx2All 2.11/REXX (Aug 7 1999) - MPx ID3 Tag -> BBS & EAs converter
 * by Anton Dushko (2:5030/611.9)
 * This stuff is free. Comments & suggestions are welcome.
 */
                                         
/* Banner */
Say "MPx2All 2.11/REXX (Aug 7 1999) - MPx ID3 Tag -> BBS & EAs converter"
Say "by Anton Dushko (2:5030/611.9)"
Say 

/* Some startup operations */
Trace Off
Signal On Syntax Name IntError
Signal On Halt Name CtrlBreak
Signal Off Error
Signal Off NoValue
Call RxFuncAdd "SysLoadFuncs", "RexxUtil", "sysloadfuncs"
Call SysLoadFuncs

/* Get command line args */
Parse Arg Params

/* Load MPxREXX */
Call MPxREXXInit

/* Defaults */
FileMask = MPxVars.DefMPxMask
BBSEnabled = MPxVars.DefBBSEnabled
EAsEnabled = MPxVars.DefEAsEnabled
BBSTemplate = MPxVars.DefBBSTemplate
EATemplate = MPxVars.DefEATemplate
SFTFileOpts = "FO"
BBSName = MPxVars.DefBBSName
FileListMode = MPxVars.DefFileListMode

/* Help */
If (Params = "/?") | (Params = "-?") | (Params = "-h") | (Params = "/h") | (Params = "--help") Then Do
  Say "MPx2All will put ID3 tag & bitstream parameters info into FILES.BBS" || "0A0D"x ||,
      "  and/or EAs" || "0A0D"x ||,
      "Usage: MPx2All [<FileMask> [/R]] [/[!]BBS] [/[!]EA] [/T:{EA|BBS}:<Tempate.TPL>]" || "0A0D"x ||,
      "               [/F:<Files.BBS>] [/[!]S]" || "0A0D"x || ,
      "  Default filemask is" MPxVars.DefMPxMask || "0A0D"x ||,
      "  /R - recurse into subdirectories" || "0A0D"x ||,
      "  /BBS - update Files.BBS" || "0A0D"x ||,
      "  /!BBS - don't update Files.BBS" || "0A0D"x ||,
      "  /EA - update EAs" || "0A0D"x ||,
      "  /!EA - don't update EAs" || "0A0D"x ||,
      "  /T:BBS - Files.BBS template name" || "0A0D"x ||,
      "  /T:EA - EAs template name" || "0A0D"x ||,
      "  /F - alternative Files.BBS name" || "0A0D"x ||,
      "  /S - one Files.BBS for all files (filelist generator mode)" || "0A0D"x ||,
      "  /!S - Files.BBS in each directory"  || "0A0D"x
  Exit 1
End   

/* Process parameters */
Do I = 1 To Words(Params)
  CurParam = Word(Params, I)
  Select 
    /* Switches */
    When (Left(CurParam, 1) = "/") | (Left(CurParam, 1) = "-") Then Do
      /* Filenames */
      If Translate(SubStr(CurParam, 2, 2)) = "F:" Then
        BBSName = SubStr(CurParam, 4)
      If Translate(SubStr(CurParam, 2, 6)) = "T:BBS:" Then
        BBSTemplate = SubStr(CurParam, 8)
      If Translate(SubStr(CurParam, 2, 5)) = "T:EA:" Then
        EATemplate = SubStr(CurParam, 7)
      /* Other */
      CurParam = Translate(SubStr(CurParam, 2))
      If CurParam = "R" Then
        SFTFileOpts = "FOS"
      If CurParam = "BBS" Then
        BBSEnabled = 1
      If CurParam = "!BBS" Then
        BBSEnabled = 0
      If CurParam = "EA" Then
        EAsEnabled = 1
      If CurParam = "!EA" Then
        EAsEnabled = 0
      If CurParam = "S" Then
        FileListMode = 1
      If CurParam = "!S" Then
        FileListMode = 0
    End
    Otherwise
      FileMask = CurParam
  End
End I
  
/* Load templates */
Call MPxREXXTemplateLoad MPxVars.DefScrTemplate
If BBSEnabled = 1 Then  
  Call MPxREXXTemplateLoad BBSTemplate
If EAsEnabled = 1 Then
  Call MPxREXXTemplateLoad EATemplate

/* Get files list */
Call SysFileTree FileMask, Files, SFTFileOpts
If Files.0 = 0 Then
Do
  Say "No files found."
  Exit 1
End

/* Main loop */
Say Files.0 "file(s) found, processing:"
Do I = 1 For Files.0

  /* Read stream info */
  Call CharOut , Files.I "- "
  If MPxREXXReadStreamInfo(Files.I) = 0 Then
  Do
    Say MPxInfo.Error
    Iterate I
  End
  
  /* Select template */
  Template = MPxREXXTemplateSelect()
  Say MPxREXXTemplateParse(Value("MPxTemplate.S_" || Template))
  
  /* Update FILES.BBS if enabled*/
  If BBSEnabled = 1 Then
  Do
    If FileListMode = 1 Then
      BBS = BBSName
    Else
      BBS = FileSpec("D", Files.I) || FileSpec("P", Files.I) || FileSpec("N", BBSName)
    Call CharOut BBS, MPxREXXTemplateParse(Value("MPxTemplate." || Template))
    Call Stream BBS, "C", "CLOSE"
  End
    
  /* Update EAs if enabled */
  If EAsEnabled = 1 Then
    Do J = 1 To Words(MPxTemplate.EAs)
      If Symbol("MPxTemplate." || Template || Word(MPxTemplate.EAs, J)) = "VAR" Then
        Call PutFormattedEA Files.I, Word(MPxTemplate.EAs, J), "ASCII", MPxREXXTemplateParse(Value("MPxTemplate." || Template || Word(MPxTemplate.EAs, J)))
    End J
End I

/* All OK ! */
Exit 0

/* Supplemental functions */

/* Put string to EA */
PutFormattedEA: Procedure
  Parse Arg File, Name, Type, Value

  /* Format EA */
  Value = Translate(Space(Translate(Value, "0D20"x, "200D"x), 0), "0D20"x, "200D"x)
  Type = Translate(Type)
  Select
    When Type = "ASCII" Then
      Value = "FDFF"x || Reverse(Right(D2C(Length(Value)), 2, "00"x)) || Value
    When Type = "MVMT" Then
      Value = "DFFF00000100FDFF"x || Reverse(Right(D2C(Length(Value)), 2, "00"x)) || Value
    Otherwise
      NOP
  End  
  
  /* Put EA */  
Return SysPutEA( File, Name, Value)

/*****************************************
 * MPxREXX functions
 * (c)1997,98 Anton Dushko (2:5030/611.9)
 */

/* MPxREXXInit - initialize MPxREXX: process INI files, load genres table,
 *   initialize MPx & codepage parameters.
 * Args: None
 * Returns: None
 */
MPxREXXInit: Procedure Expose MPx. MPxVars. CPTable.

  /* Get location of MPxREXX */
  Parse Source . . SrcName
  MPxREXXPath = FileSpec("D", SrcName) || FileSpec("P", SrcName)
  MPxREXXCmd = Left(FileSpec("N", SrcName), LastPos(".", FileSpec("N", SrcName)) - 1)
  
  /* Processing <Caller>.INI */
  INILine = 0
  INIFile = MPxREXXPath || MPxREXXCmd || ".INI"
  Call Stream INIFile, "C", "OPEN READ"
  Do Until Stream(INIFile) <> "READY"
    INILine = INILine + 1
    Parse Value Translate(LineIn(INIFile), " ", "9"x) With Variable Value
    If DataType(Variable, "S") = 1 Then
      Interpret "MPxVars." || Strip(Variable) || "=" || Strip(Value)
  End
  Call Stream INIFile, "C", "CLOSE"
  Signal On Syntax Name IntError
  Drop Variable Value SrcName
  
  /* Initializing genres list */
  Call Stream MPxREXXPath || "Genres.LST", "C", "OPEN READ"
  Do Until Stream(MPxREXXPath || "Genres.LST") <> "READY"
    Parse Value LineIn(MPxREXXPath || "Genres.LST") With Index Name
    If DataType(Index, "N") = 1 Then
      MPx.Genre.Index = Name
  End
  Call Stream MPxREXXPath || "Genres.LST", "C", "CLOSE"
  Drop Index Name

  /* Initializing mode names list */
  MPx.CMode.0 = "Stereo"
  MPx.CMode.1 = "J-Stereo"
  MPx.CMode.2 = "Dual-Chan"
  MPx.CMode.3 = "Mono"

  /* Initializing deemph. mode names list */
  MPx.DeEmph.0 = "None"
  MPx.DeEmph.1 = "50/15 ms"
  MPx.DeEmph.2 = "?"
  MPx.DeEmph.3 = "CIIT j.17"

  /* Initializing sampling freqs list */
  MPx.SFreq.1 = "44.1   48 32 ?"
  MPx.SFreq.2 = "22.05  24 16 ?"
  MPx.SFreq.4 = "11.025  ?  8 ?"

  /* Initializing bitrates list */
  MPx.BRate.1.1 = "0  32  64  96 128 160 192 224 256 288 320 352 384 416 448 ?"
  MPx.BRate.1.2 = "0  32  48  56  64  80  96 112 128 160 192 224 256 320 384 ?"
  MPx.BRate.1.3 = "0  32  40  48  56  64  80  96 112 128 160 192 224 256 320 ?"
  MPx.BRate.2.1 = "0  32  48  56  64  80  96 112 128 144 160 176 192 224 256 ?"
  MPx.BRate.2.2 = "0   8  16  24  32  40  48  56  64  80  96 112 128 144 160 ?"
  MPx.BRate.2.3 = MPx.BRate.2.2
  MPx.BRate.4.1 = MPx.BRate.2.1
  MPx.BRate.4.2 = MPx.BRate.2.2
  MPx.BRate.4.3 = MPx.BRate.2.3

  /* Initializing codepage detection & conversion tables */  
  CPTable.Conv.866 = "񦧨"
  CPTable.Conv.1251 = "Ũ"
  CPTable.Mask.866 = X2C(B2X(Translate(Translate(XRange("80"x, "FF"x), Copies("1", Length(CPTable.Conv.866)), CPTable.Conv.866), Copies("0", 128), XRange("80"x, "FF"x))))
  CPTable.Mask.1251 = X2C(B2X(Translate(Translate(XRange("80"x, "FF"x), Copies("1", Length(CPTable.Conv.1251)), CPTable.Conv.1251), Copies("0", 128), XRange("80"x, "FF"x))))
  CPTable.Mask.Undef = BitAnd(CPTable.Mask.866, CPTable.Mask.1251)
  CPTable.Mask.866 = BitXor(CPTable.Mask.866, CPTable.Mask.Undef)
  CPTable.Mask.1251 = BitXor(CPTable.Mask.1251, CPTable.Mask.Undef)
  Drop CPTable.Mask.Undef

Return

/* MPxREXXReadStreamInfo - reads MPx stream info & fills MPxInfo structure
 *   (ID3 strings are converted to MPxVars.Codepage cp).
 * Args: MPxFile - File to process
 * Returns: 
 *   0 - all OK
 *   1 - error (check description in MPxInfo.Error)
 * Notes: MPxInfo structure description:
 *   MPxInfo.Error - error text ("" if no error)
 *   MPxInfo.FileName - name of file
 *   MPxInfo.MPEG - MPEG version
 *   MPxInfo.Layer - Layer #
 *   MPxInfo.BitRate - Stream bitrate
 *   MPxInfo.Frequrency - Stream samplerate
 *   MPxInfo.Mode - Stream mode
 *   MPxInfo.ModeExt - Stream mode extention
 *   MPxInfo.DeEmph - Stream deemphasis
 *   MPxInfo.CRC         \
 *   MPxInfo.Original     |- Stream flags
 *   MPxInfo.Copyrighted  |
 *   MPxInfo.Private     /
 *   MPxInfo.Flags - above flags in verbal presentation
 *   MPxInfo.FileSize - size of MPx file
 *   MPxInfo.JunkSize - size of junk in MPx file
 *   MPxInfo.Time - MPx playing time
 *   MPxInfo.ID3 - nonzero if ID3 exists (actually position of ID3 TAG in file)
 *   MPxInfo.ID3.Title   \
 *   MPxInfo.ID3.Artist   |
 *   MPxInfo.ID3.Album    |- Info from ID3
 *   MPxInfo.ID3.Year     |
 *   MPxInfo.ID3.Comment /
 *   MPxInfo.ID3.Genre - Verbal presentation of genre
 *   MPxInfo.ID3.GenreCode - Genre code
 */
MPxREXXReadStreamInfo: Procedure Expose MPx. MPxVars. MPxInfo. CPTable.
  Arg MPxFile

  /* Clear information structure */
  Drop MPxInfo.
  
  /* Opening file */
  RC = Stream(MPxFile, "C", "OPEN READ")
  If RC <> "READY:" Then
  Do
    MPxInfo.Error = "Can't open file for reading:" RC
    Return 0
  End  

  /* Looking for syncword */
  FrameHeader = CharIn(MPxFile,, MPxVars.MaxJunkSize)
  Do Until Left(X2B(C2X(SubStr(FrameHeader, JSize + 1, 1))), 3) = "111"
    JSize = Pos(X2C("FF"), FrameHeader)
    If JSize = 0 Then
      Leave
  End 
  
  /* Junk? */
  If JSize = 0 Then Do
    MPxInfo.JunkSize = MPxVars.MaxJunkSize
    MPxInfo.Error = "Syncword not found"
    Call Stream MPxFile, "C", "CLOSE"
    Return 0
  End
     
  /* Analysing frame header */
  FrameHeader = X2B(C2X(SubStr(FrameHeader, JSize, 4)))
  MPxInfo.FileName = FileSpec("N", MPxFile)
  JSize = JSize - 1
  MPxInfo.JunkSize = JSize
  MPxInfo.MPEG = 4 - B2X(SubStr(FrameHeader, 12, 2))
  MPxInfo.Layer = 4 - B2X(SubStr(FrameHeader, 14, 2))
  If (MPxInfo.Layer = 4) | (MPxInfo.MPEG = 3) Then
    MPxInfo.BitRate = "?"
  Else  
    Interpret "MPxInfo.BitRate = Word(MPx.BRate." || MPxInfo.MPEG || "." || MPxInfo.Layer || "," || X2D(B2X(SubStr(FrameHeader, 17, 4))) + 1 || ")"
  Interpret "MPxInfo.Frequrency = Word(MPx.SFreq." || MPxInfo.MPEG || "," || B2X(SubStr(FrameHeader, 21, 2)) + 1 || ")"
  Interpret "MPxInfo.Mode = MPx.CMode." || B2X(SubStr(FrameHeader, 25, 2))
  MPxInfo.CRC = 1 - SubStr(FrameHeader, 16, 1)
  MPxInfo.Private = SubStr(FrameHeader, 24, 1)
  MPxInfo.Copyrighted = SubStr(FrameHeader, 29, 1)
  MPxInfo.Oririnal = SubStr(FrameHeader, 30, 1)
  MPxInfo.ModeExt = B2X(SubStr(FrameHeader, 27, 2))
  Interpret "MPxInfo.DeEmph = MPx.DeEmph." || B2X(SubStr(FrameHeader, 31, 2))
  If MPxInfo.MPEG = 4 Then 
    MPxInfo.MPEG = "2.5"

  /* Fill flags */
  MPxInfo.Flags = ""
  If MPxInfo.CRC = 1 Then
    MPxInfo.Flags = "CRC-Prot"
  If MPxInfo.Private =  1 Then 
    MPxInfo.Flags = MPxInfo.Flags || " Private"
  If MPxInfo.Copyrighted = 1  Then
    MPxInfo.Flags = MPxInfo.Flags || " Copyrighted"
  If MPxInfo.Oririnal = 1 Then
    MPxInfo.Flags = MPxInfo.Flags || " Original"
  MPxInfo.Flags = Space(MPxInfo.Flags)

  /* Reading ID3 tag & closing file (kluged for Regina) */
  MPxInfo.ID3 = Stream(MPxFile, "C", "QUERY SIZE") - 128
  Tag = CharIn(MPxFile, MPxInfo.ID3 + 1, 128)
  Call Stream MPxFile, "C", "CLOSE"
      
  /* Check ID3 */
  If Left(Tag, 3) = "TAG" Then
  Do
    /* Extract components from ID3 */
    JSize = JSize + 128
    I = Pos("00"x, SubStr(Tag, 4, 30))
    If I = 0 Then I = 31
    MPxInfo.ID3.Title = MPxREXXCPConv(MPxVars.CodePage, Space(Left(SubStr(Tag, 4, 30), I - 1)))
    I = Pos("00"x, SubStr(Tag, 34, 30))
    If I = 0 Then I = 31
    MPxInfo.ID3.Artist = MPxREXXCPConv(MPxVars.CodePage, Space(Left(SubStr(Tag, 34, 30), I - 1)))
    I = Pos("00"x, SubStr(Tag, 64, 30))
    If I = 0 Then I = 31
    MPxInfo.ID3.Album = MPxREXXCPConv(MPxVars.CodePage, Space(Left(SubStr(Tag, 64, 30), I - 1)))
    I = Pos("00"x, SubStr(Tag, 94, 4))
    If I = 0 Then I = 5
    MPxInfo.ID3.Year = Strip(Space(Left(SubStr(Tag, 94, 4), I - 1)))
    I = Pos("00"x, SubStr(Tag, 98, 30))
    If I = 0 Then I = 31
    MPxInfo.ID3.Comment = MPxREXXCPConv(MPxVars.CodePage, Space(Left(SubStr(Tag, 98, 30), I - 1)))
    MPxInfo.ID3.GenreCode = C2D(SubStr(Tag, 128, 1))
    Interpret "MPxInfo.ID3.Genre = MPx.Genre." || MPxInfo.ID3.GenreCode
    If MPxInfo.ID3.Genre = "MPx.Genre." || MPxInfo.ID3.GenreCode Then
      MPxInfo.ID3.Genre = MPx.Genre.255
  End
  Else
    MPxInfo.ID3 = 0

  /* Calculate bitstream time */
  MPxInfo.FileSize = Stream(MPxFile, "C", "QUERY SIZE")
  If MPxInfo.BitRate <> "?" Then
  Do
    Seconds = (MPxInfo.FileSize - JSize) / (MPxInfo.BitRate * 125)
    MPxInfo.Time = Seconds % 60 || ":" || Right(Space(Format(Seconds // 60, 2, 0)), 2, "0")
  End
  Else
    MPxInfo.Time = "--:--"

  /* All OK */  
  MPxInfo.Error = ""
Return 1

/* MPxREXXCPConv - Convert string to given CP 
 * Args:
 *   CP - Codepage (0, 866 or 1251)
 *   String - String to convert
 * Returns: Converted string
 */
MPxREXXCPConv: Procedure Expose CPTable.
  Parse Arg CP, String
  
  /* Checking if CP valid */
  If Symbol("CPTable.Conv."CP) <> "VAR" Then 
    Return String
  
  /* Autodetecting CP */
  StringMask = X2C(B2X(Translate(Translate(XRange("80"x,"FF"x), Copies("1", Length(String)), String), Copies("0", 128), XRange("80"x,"FF"x))))
  Is866 = Pos("1",X2B(C2X(BitAnd(StringMask, CPTable.Mask.866))))
  Is1251 = Pos("1",X2B(C2X(BitAnd(StringMask, CPTable.Mask.1251))))
  
  /* Converting */
  Select
    When (Is866 <> 0) & (Is1251 = 0) Then
      String = Translate(String, CPTable.Conv.CP, CPTable.Conv.866)
    When (Is866 = 0) & (Is1251 <> 0) Then
      String = Translate(String, CPTable.Conv.CP, CPTable.Conv.1251)
    Otherwise 
      NOP
  End  
Return String

/* MPxREXXShut - shutdown MPxREXX: drop all MPx & codepage parameters &
 *   MPxREXX variables.
 * Args: None
 * Returns: None
 */
MPxREXXShut: Procedure Expose MPx. MPxVars. CPTable.
  Drop MPx. MPxVars. CPTable.
Return 

/* MPxREXXTemplateLoad - Load template file for use with MPxREXX
 * Args: TPL filename
 * Returns: None
 */
MPxREXXTemplateLoad: Procedure Expose MPxTemplate.
  Arg TPLFile
  
  RC = Stream(TPLFile, "C", "OPEN READ")
  If RC <> "READY:" Then
    Signal TPLError
    
  /* Load & close TPL file */
  AllTemplates = CharIn(TPLFile,, Stream(TPLFile, "C", "QUERY SIZE"))
  Call Stream TPLFile, "C", "CLOSE"
  
  /* Process it */
  M = 0
  MPxTemplate.EAs = ""
  Do Forever
    /* Find template in file */
    N = Pos("%+", AllTemplates, M + 1)
    If N = 0 Then
      Leave
    M = Pos("%-%", AllTemplates, N + 1)
    If M = 0 Then
      M = Length(AllTemplates)
    K = Pos("%", AllTemplates, N + 1)
    
    /* Gather template body & name */
    Template = SubStr(AllTemplates, K + 1, M - K - 1)
    Name = SubStr(AllTemplates, N + 2, K - N - 2)
    
    /* Build EA list */
    PntPos = Pos(".", Name)
    If PntPos <> 0 Then
      If Pos(SubStr(Name, PntPos), MPxTemplate.EAs) = 0 Then
         MPxTemplate.EAs = MPxTemplate.EAs || " " || SubStr(Name, PntPos)

    /* Process wildcarded template name */
    WildMask = Translate(Name, Copies(" ", C2D("9") - C2D(" ") + 1) || "1" || Copies(" ", C2D("z") - C2D("A") + 1) , XRange(" ", "9") || "?" || XRange("A", "z"))
    WildChars = Space(WildMask, 0)
    WildCharsNum = Length(WildChars)
    WildChars = X2D(B2X("0" || WildChars)) + 1
    J = 0
    Do I = 2 To (WildCharsNum + 1)
      J = Pos("1", WildMask, J + 1)
      Name = Overlay(I, Name, J, 1)
    End I
    Do I = 0 For WildChars
      Interpret("MPxTemplate." || Translate(Name, Right(X2B("00" || D2X(I)), WildCharsNum), XRange(2, WildCharsNum + 2)) || "= Template") 
    End I
  End
Return 

/* Find corresponding template name */    
MPxREXXTemplateSelect: Procedure Expose MPxInfo.
  If MPxInfo.ID3 = 0 Then
    ID3Mask = "NoID3"
  Else 
    ID3Mask = (Length(MPxInfo.ID3.Title) <> 0) || (Length(MPxInfo.ID3.Artist) <> 0) || (Length(MPxInfo.ID3.Album) <> 0) || (Length(MPxInfo.ID3.Year) <> 0) || (Length(MPxInfo.ID3.Comment) <> 0)
Return ID3Mask
  
/* Parse a template */
MPxREXXTemplateParse: Procedure Expose MPxInfo.
  Parse Arg Template  
  
  Result = ""
  Do I = 1
    J = Pos("%", Template, I)
    If J = 0 Then
      J = Length(Template) + 1
    Result = Result || SubStr(Template, I, J - I)
    I = J + 1
    If I > Length(Template) Then
      Leave I
    J = Pos("%", Template, I)
    If J = 0 Then 
      Leave I
    Parse Value SubStr(Template, I, J - I) With Var","Len","Align","Pad .
    If Align = "" Then
      Align = "L"
    If Pad = "" Then
      Pad = " "
    Var = Translate(Var)
    If Symbol("MPxInfo." || Var) = "VAR" Then
    Select
      When Len = "" Then
        Result = Result || MPxInfo.Var
      When Align = "L" Then
        Result = Result || Left(MPxInfo.Var, Len, Pad)
      When Align = "R" Then
        Result = Result || Right(MPxInfo.Var, Len, Pad)
      When Align = "C" Then
        Result = Result || Center(MPxInfo.Var, Len, Pad)
    End
    I = J
  End I
Return Result

/* MPxREXXTemplateDrop - drops templates
 * Args: None
 * Returns: None
 */
MPxREXXTemplateDrop: Procedure Expose MPxTemplate.
  Drop MPxTemplate.
Return

/* MPxREXX signal handlers */

INIError:
  Say
  Say "INI error in "INIFile", line "INILine
  Exit -11
  
TPLError:
  Say
  Say "TPL error in "TPLFile "-" RC
  Exit -12

/* End of MPxREXX 
 *****************/


/* Supplemental */

/* Regina doesn't have FileSpec function, here's substitution: */
FileSpec: Procedure
  Parse Arg Option, Path

  Option = Translate(Left(Option, 1))
  SCPos = Pos(":", Path)
  BSLPos = LastPos("\", Path)
  Select
    /* Drive */
    When Option = "D" Then 
      If SCPos \= 0 Then
        Return Left(Path, SCPos)
      Else
        Return ""
    /* Path */
    When Option = "P" Then 
      If BSLPos \= 0 Then
        Return SubStr(Path, SCPos + 1, BSLPos - SCPos)
      Else
        Return ""
    /* Name */
    When Option = "N" Then 
      Return SubStr(Path, Max(SCPos, BSLPos) + 1)
    Otherwise
      Return ""
  End
Return ""


/* Signal handlers */

IntError:
  Say
  Say "Internal error" RC "at line" SIGL
  Exit -5

CtrlBreak:
  Say
  Say "Ctrl-Break pressed!"
  Exit -1

/* End of MPx2All.CMD */
